﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"

using namespace CPPTester;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Navigation;
using namespace WinRTComponent;
using namespace Platform::Collections;
using namespace Windows::Foundation::Collections;
using namespace Concurrency;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

MainPage::MainPage()
{
	InitializeComponent();

	c = ref new Component();
}

/// <summary>
/// Invoked when this page is about to be displayed in a Frame.
/// </summary>
/// <param name="e">Event data that describes how this page was reached.  The Parameter
/// property is typically used to configure the page.</param>
void MainPage::OnNavigatedTo(NavigationEventArgs^ e)
{
	(void) e;	// Unused parameter

	// simple types
	int x = c->Add(1,2);
	float y = c->Sub(2, 1);

	// WinRT type conversions
	c->Navigate(ref new Uri("http://channel9.msdn.com/coding4fun/"));
	DateTime dt = c->GetTime();

	// collections
	int numbers[] = { 1, 2, 3, 4, 5, 6 };
	Vector<int>^ vNumbers = ref new Vector<int>(numbers);
	float average = c->Average(vNumbers);

	Map<String^, int>^ map = ref new Map<String^, int>();
	map->Insert("hi", 1);
	c->SetDictionary(map);

	PropertySet^ ps = ref new PropertySet();
	ps->Insert("hi", 1);
	c->SetIPropertySet(ps);

	// arrays
	Array<int>^ aNumbers = ref new Array<int>(numbers, 6);
	float averageArray = c->AverageArray(aNumbers);

	// these don't work (intentionally)
	//c->ArrayReadOnly(aNumbers, 10);
	//c->ArrayWriteOnly(aNumbers, 10);
	//c->ArrayClone(aNumbers, 10);

	// this is a proper call, but the array is re-intialized
	c->ArrayOut(&aNumbers, 10);

	aNumbers = c->ArrayCloneReturn(aNumbers, 10);

	// overloads
	c->Overload("string");
	c->Overload(1.0f);
	c->Overload(1);

	// events
	c->MyEvent += ref new EventHandler<MyEventArgs^>([](Object^sender, MyEventArgs^args)
	{
		int x = args->Field;
		OutputDebugString(L"event");
	});
	c->RaiseEvent();

	// exceptions
	try
	{
		c->ThrowFileNotFoundException();
	}
	catch(Exception^ ex)
	{
		OutputDebugString(ex->Message->Data());
	}

	try
	{
		c->ThrowNullReferenceException();
	}
	catch(Exception^ ex)
	{
		OutputDebugString(ex->Message->Data());
	}

	try
	{
		c->ThrowCOMException();
	}
	catch(Exception^ ex)
	{
		OutputDebugString(ex->Message->Data());
	}

	try
	{
		c->ThrowMyException();
	}
	catch(Exception^ ex)
	{
		OutputDebugString(ex->Message->Data());
	}

	// async
	create_task(c->AsyncAction()).then([=]()
	{
		OutputDebugString(L"done\r\n");
		return c->AsyncOperation();
	}).then([=](String^ s)
	{
		OutputDebugString(s->Data());
		OutputDebugString(L"\r\n");
		return c->AsyncActionCancellation();
	}).then([=]()
	{
		OutputDebugString(L"done\r\n");
		return c->AsyncOperationCancellation();
	}).then([=](String^ s)
	{
		OutputDebugString(s->Data());
		OutputDebugString(L"\r\n");
		return c->AsyncActionProgress();
	}).then([=]()
	{
		OutputDebugString(L"done\r\n");
		return c->AsyncOperationCancellation();
	}).then([=](String^ s)
	{
		OutputDebugString(s->Data());
		OutputDebugString(L"\r\n");
	}).then([=](task<void> t)
	{
		OutputDebugString(L"all tasks complete\r\n");
		t.get();
	});
}
