﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Foundation.Metadata;

namespace WinRTComponent
{
	public sealed class MyEventArgs
	{
		public int Field { get; set; }
	}

	public sealed class Component
	{
		public event EventHandler<MyEventArgs> MyEvent;

		/////////////////////////////
		// basic types
		public int Add(int x, int y)
		{
			return x + y;
		}

		public Single Sub(Single x, Single y)
		{
			return x - y;
		}
		// basic types
		/////////////////////////////


		/////////////////////////////
		// WinRT vs. .NET types
		public void Navigate(Uri uri)
		{
			// TODO: navigate :)
		}

		public DateTimeOffset GetTime()
		{
			return DateTime.Now;
		}
		// WinRT vs. .NET types
		/////////////////////////////


		/////////////////////////////
		// Collections
		public float Average(IList<int> values)
		{
			return  (float)values.Aggregate((acc, cur) => acc + cur) / values.Count;
		}

		// JavaScript can't instantiate anything but PropertySet, which is IMap<string, object>
		// You must create/return your own implmentation for types which aren't this!
		public IDictionary<string, int> GetMap()
		{
			return new Dictionary<string,int>();
		}

		public void SetDictionary(IDictionary<string, int> dict)
		{
			dict["hi"] = 3;
		}

		public void SetIPropertySet(IPropertySet ps)
		{
			ps["hi"] = 3;
		}
		// Collections
		/////////////////////////////


		/////////////////////////////
		// Arrays
		public Single AverageArray([ReadOnlyArray]int[] values)
		{
			return (Single)values.Aggregate((acc, cur) => acc + cur) / values.Length;
		}

		// this works in managed only
		public void ArrayReadOnly([ReadOnlyArray]int[] values, int multiplier)
		{
			for(int i = 0; i < values.Length; i++)
				values[i] *= multiplier;
		}
		// this works in managed only
		public void ArrayWriteOnly([WriteOnlyArray]int[] values, int multiplier)
		{
			for(int i = 0; i < values.Length; i++)
				values[i] *= multiplier;
		}
		// this works in managed only
		public void ArrayOut(out int[] values, int multiplier)
		{
			// out parameters are assumed marked wtih WriteOnlyArrayAttribute
			// can't assume anything about the values array, so we must initialize it ourselves
			values = new[] { 1, 2 };
			for(int i = 0; i < values.Length; i++)
				values[i] *= multiplier;
		}
		// this (intentionally) doesn't work :)
		public void ArrayClone([ReadOnlyArray]int[] values, int multiplier)
		{
			int[] clone = (int[])values.Clone();
			for(int i = 0; i < clone.Length; i++)
				clone[i] *= multiplier;
			values = clone;
		}
		// this is the proper way to do this cross-platform
		public int[] ArrayCloneReturn([ReadOnlyArray]int[] values, int multiplier)
		{
			// but returning the array does!
			int[] clone = (int[])values.Clone();
			for(int i = 0; i < clone.Length; i++)
				clone[i] *= multiplier;
			return clone;
		}
		// Arrays
		/////////////////////////////


		/////////////////////////////
		// Method overloads
		[DefaultOverload]	// without this, the compiler will complain
		public void Overload(string s)	{ Debug.WriteLine("string"); }
		public void Overload(int i)		{ Debug.WriteLine("int"); }
		public void Overload(float f)	{ Debug.WriteLine("float"); }	
		// Method overloads
		/////////////////////////////


		/////////////////////////////
		// Events
		public void RaiseEvent()
		{
			if(MyEvent != null)
				MyEvent(this, new MyEventArgs { Field = 1 });
		}
		// Events
		/////////////////////////////


		/////////////////////////////
		// Exceptions
		public void ThrowFileNotFoundException()
		{
			throw new FileNotFoundException("Custom error message");
		}

		public void ThrowNullReferenceException()
		{
			throw new NullReferenceException("Custom error message");
		}

		public void ThrowCOMException()
		{
			throw new COMException("My COM Message", -1);
		}

		public void ThrowMyException()
		{
			throw new MyException();
		}
		// Exceptions
		/////////////////////////////


		/////////////////////////////
		// Async operations
		public IAsyncAction AsyncAction()
		{
			return Task.Run(async () => await Task.Delay(1000)).AsAsyncAction();
		}

		public IAsyncOperation<string> AsyncOperation()
		{
			return Task.Run(async () =>
			{
				await Task.Delay(1000);
				return Environment.TickCount.ToString();
			}).AsAsyncOperation();
		}

		public IAsyncAction AsyncActionCancellation()
		{
			return AsyncInfo.Run(token => Task.Run(async () => await Task.Delay(1000) ));
		}

		public IAsyncOperation<string> AsyncOperationCancellation()
		{
			return AsyncInfo.Run(token => 
				Task.Run(async () => 
				{
					await Task.Delay(1000);
					return Environment.TickCount.ToString();
				}
			));
		}

		public IAsyncActionWithProgress<int> AsyncActionProgress()
		{
			return AsyncInfo.Run((CancellationToken token, IProgress<int> progress) => 
				Task.Run(async () => 
				{
					progress.Report(0);
					await Task.Delay(1000);
					progress.Report(100);
				}));
		}

		public IAsyncOperationWithProgress<string, int> AsyncOperationProgress()
		{
			return AsyncInfo.Run((CancellationToken token, IProgress<int> progress) => 
				Task.Run(async () => 
				{
					progress.Report(0);
					await Task.Delay(1000);
					progress.Report(100);
					return Environment.TickCount.ToString();
				}));
		}
		// Async operations
		/////////////////////////////
	}

	class MyException : Exception
	{
		public MyException() { HResult = -0x1234; }
	}
}
